@@ -1,5 +1,50 @@
 """
-.. include:: ../../docs/advanced/display.md
+# Display
+
+Where the advanced API really shines is in displaying nodes.
+There are basically two ways of displaying a node:
+
+* *plain*: just the associated text of a node, or if that would be too much,
+  an identifying label of that node (e.g. for books, chapters and lexemes).
+* *pretty*: a display of the internal structure of the textual object a node
+  stands for. That structure is adorned with relevant feature values.
+
+These display methods are available for nodes, tuples of nodes, and iterables
+of tuples of nodes (think: query results).
+The names of these methods are
+
+* `plain`, `plainTuple`, and `table`;
+* `pretty`, `prettyTuple` and `show`.
+
+In plain and pretty displays, certain parts can be *highlighted*, which is
+good for displaying query results where the parts that correspond directly to the
+search template are highlighted.
+
+## Display parameters
+
+There is a bunch of parameters that govern how the display functions arrive at their
+results. You can pass them as optional arguments to these functions,
+or you can set up them in advance, and reset them to their original state
+when you are done.
+
+All calls to the display functions look for the values for these parameters in the
+following order:
+
+* optional parameters passed directly to the function,
+* values as set up by previous calls to `displaySetup()`,
+* corpus dependent default values configured by the advanced API.
+
+See `tf.advanced.options` for a list of display parameters.
+
+## Rendering
+
+Both `pretty` and `plain` are implemented as a call to the
+`tf.advanced.render.render` function.
+
+## See also
+
+All about the nature and implementation of the display algorithm is in
+`tf.about.displaydesign`.
 """
 
 
@@ -8,7 +53,7 @@ import types
 
 from ..parameters import DOWNLOADS, SERVER_DISPLAY, SERVER_DISPLAY_BASE
 from ..core.helpers import mdEsc
-from .helpers import getResultsX, tupleEnum, RESULT, dh, showDict
+from .helpers import getRowsX, tupleEnum, RESULT, dh, showDict
 from .condense import condense, condenseSet
 from .highlight import getTupleHighlights
 from .options import Options
@@ -54,6 +99,7 @@ def displayApi(app, silent):
     app.pretty = types.MethodType(pretty, app)
     app.unravel = types.MethodType(unravel, app)
     app.loadCss = types.MethodType(loadCss, app)
+    app.getCss = types.MethodType(getCss, app)
     app.displayShow = types.MethodType(displayShow, app)
     app.displaySetup = types.MethodType(displaySetup, app)
     app.displayReset = types.MethodType(displayReset, app)
@@ -142,17 +188,51 @@ def displayReset(app, *options):
 
 
 def loadCss(app):
-    """The CSS is looked up and then loaded into a notebook if we are not
-    running in the TF browser,
-    else the CSS is returned.
+    """Load the CSS for this app.
+
+    If we are in the TF-browser, the generic CSS is already provided, we only
+    need to respond with the app-specific CSS: we return it as string.
+    The flag `app._browse` is used to steer us into this case.
+
+    Otherwise, we collect the complete CSS code from Text-Fabric and the app,
+    and we add a piece to override some of the notebook CSS for tables,
+    which specify a table layout with right aligned cell contents by default.
+
+    We then load the resulting CSS into the notebook.
+
+    Returns
+    -------
+    None | string
+        When in the TF browser, the app-dependent CSS is returned.
+        Otherwise, nothing is returned, but the complete CSS is displayed as HTML in the notebook.
     """
 
     _browse = app._browse
     aContext = app.context
-    css = aContext.css
+    appCss = aContext.css
 
     if _browse:
-        return css
+        return appCss
+
+    css = getCss(app)
+    dh(css)
+
+
+def getCss(app):
+    """Export the CSS for this app.
+
+    We collect the complete CSS code from Text-Fabric and the app,
+    and we add a piece to override some of the notebook CSS for tables,
+    which specify a table layout with right aligned cell contents by default.
+
+    Returns
+    -------
+    None | string
+        CSS code, including a surrounding `<style>` element.
+    """
+
+    aContext = app.context
+    appCss = aContext.css
 
     cssPath = (
         f"{os.path.dirname(os.path.dirname(os.path.abspath(__file__)))}"
@@ -167,7 +247,7 @@ def loadCss(app):
         "tr.tf.ltr, td.tf.ltr, th.tf.ltr { text-align: left ! important;}\n"
         "tr.tf.rtl, td.tf.rtl, th.tf.rtl { text-align: right ! important;}\n"
     )
-    dh("<style>" + tableCss + genericCss + css + "</style>")
+    return f"<style>{tableCss}{genericCss}{appCss}</style>"
 
 
 def export(app, tuples, toDir=None, toFile="results.tsv", **options):
@@ -175,15 +255,17 @@ def export(app, tuples, toDir=None, toFile="results.tsv", **options):
 
     !!! hint "Examples"
         See for detailed examples the
-        [exportExcel](https://nbviewer.jupyter.org/github/annotation/tutorials/blob/master/bhsa/exportExcel.ipynb)
+        [exportExcel (bhsa)](https://nbviewer.jupyter.org/github/annotation/tutorials/blob/master/bhsa/exportExcel.ipynb)
         and
-        [exportExcel](https://nbviewer.jupyter.org/github/annotation/tutorials/blob/master/oldbabylonian/exportExcel.ipynb)
+        [exportExcel (oldbabylonian)](https://nbviewer.jupyter.org/github/annotation/tutorials/blob/master/oldbabylonian/exportExcel.ipynb)
         notebooks.
 
     Parameters
     ----------
     tuples: iterable of tuples of integer
         The integers are the nodes, together they form a table.
+        The table maybe uniform or not uniform,
+        which matters to the output. See below.
     toDir: string, optional `None`
         The destination directory for the exported file.
         By default it is your Downloads folder.
@@ -225,6 +307,11 @@ def export(app, tuples, toDir=None, toFile="results.tsv", **options):
     A file *toFile* in directory *toDir* with the following content:
 
     There will be a row for each tuple.
+
+    If the input tuples are *uniform*, i.e. each tuple has the
+    same number of nodes, and nodes in the same column have the same node types,
+    then the result table has the following layout:
+
     The columns are:
 
     * **R** the sequence number of the result tuple in the result list
@@ -242,6 +329,14 @@ def export(app, tuples, toDir=None, toFile="results.tsv", **options):
       where these features have been declared by a previous
       displaySetup(tupleFeatures=...)`
 
+    If the input tuples are not uniform, the layout is more primitive.
+    There will be no header column, because the number of columns may vary per row.
+    A row contains the successive information of all nodes in a tuple.
+    Depending of the type of each node you get a number of columns of section information.
+    Then follow two columns with the node and the node type.
+    Depending on the type of the node, there follows a column with the text of the node.
+    No additional features are produced.
+
     !!! caution "Encoding"
         The exported file is written in the `utf_16_le` encoding.
         This ensures that Excel can open it without hassle, even if there
@@ -267,7 +362,7 @@ def export(app, tuples, toDir=None, toFile="results.tsv", **options):
             os.makedirs(toDir, exist_ok=True)
     toPath = f"{toDir}/{toFile}"
 
-    resultsX = getResultsX(app, tuples, tupleFeatures, condenseType, fmt=fmt,)
+    resultsX = getRowsX(app, tuples, tupleFeatures, condenseType, fmt=fmt)
 
     with open(toPath, "w", encoding="utf_16_le") as fh:
         fh.write(
@@ -455,7 +550,7 @@ def plainTuple(
             if _browse
             else app.webLink(passageNode, _asString=True)
         )
-        passageRef = f'<span class="section ltr">{passageRef}</span>'
+        passageRef = f'<span class="tfsechead ltr">{passageRef}</span>'
     else:
         passageRef = ""
 
@@ -482,7 +577,7 @@ def plainTuple(
             passageAtt = ""
 
         plainRep = "".join(
-            "<span>"
+            '<span class="col">'
             + mdEsc(
                 app.plain(
                     n,
@@ -702,7 +797,9 @@ def prettyTuple(app, tup, seq, item=RESULT, **options):
         html = []
     for t in sorted(containers, key=sortKey):
         h = app.pretty(
-            t, highlights=highlights, **display.consume(options, "highlights"),
+            t,
+            highlights=highlights,
+            **display.consume(options, "highlights"),
         )
         if _browse:
             html.append(h)
