@@ -1,5 +1,10 @@
 """
-.. include:: ../../docs/advanced/unravel.md
+# Unravel
+
+Unraveling means to transform a neighbourhood of nodes into a tree of node fragment.
+That sounds simple, but quite a few ideas have to work together to make it work.
+
+It is described at length in `tf.about.displaydesign`.
 """
 
 from collections import namedtuple
