@@ -1,5 +1,32 @@
 """
-.. include:: ../../docs/compose/combine.md
+# Combine
+
+## Usage
+
+```python
+from tf.compose import combine
+combine(
+  (
+      location1,
+      location2,
+  ),
+  targetLocation,
+)
+```
+
+```
+combine(
+  (
+      (name1, location1),
+      (name2, location2),
+  ),
+  targetLocation,
+  componentType=None,
+  componentFeature=None,
+  featureMeta=None,
+  **otext,
+)
+```
 """
 
 import os
@@ -214,7 +241,8 @@ def combine(
 
     if not dirEmpty(targetLocation):
         error(
-            f"Output directory is not empty. Clean it or remove it or choose another location",
+            "Output directory is not empty."
+            " Clean it or remove it or choose another location",
             tm=False,
         )
         return False
@@ -348,7 +376,7 @@ def combine(
         commonTypes = set(slotTypes) & set(nodeTypesComp)
         if len(commonTypes):
             error(
-                f"Some node types are slots in one source and non slots in another",
+                "Some node types are slots in one source and non slots in another",
                 tm=False,
             )
             error(", ".sorted(commonTypes), tm=False)
