@@ -1,5 +1,138 @@
 """
-.. include:: ../../docs/convert/walker.md
+# Walker
+
+You can convert a dataset to TF by writing a function that walks through it.
+
+That function must trigger a sequence of actions when reading the data.
+These actions drive Text-Fabric to build a valid Text-Fabric dataset.
+Many checks will be performed.
+
+!!! hint "to and from MQL"
+    If your source is MQL, you are even better off: Text-Fabric has a
+    module to import from MQL and to export to MQL.
+    See `tf.fabric.Fabric.importMQL` and `tf.fabric.Fabric.exportMQL`.
+
+## Set up
+
+Here is a schematic set up of such a conversion program.
+
+```python
+from tf.fabric import Fabric
+from tf.convert.walker import CV
+
+TF = Fabric(locations=OUT_DIR)
+cv = CV(TF)
+
+def director(cv):
+  # your code to unwrap your source data and trigger
+  # the generation of TF nodes, edges and features
+
+slotType = 'word'  # or whatever you choose
+otext = {  # dictionary of config data for sections and text formats
+    ...
+}
+generic = {  # dictionary of metadata meant for all features
+    ...
+}
+intFeatures = {  # set of integer valued feature names
+    ...
+}
+featureMeta = {  # per feature dicts with metadata
+   ...
+}
+
+good = cv.walk(
+    director,
+    slotType,
+    otext=otext,
+    generic=generic,
+    intFeatures=intFeatures,
+    featureMeta=featureMeta,
+    warn=True,
+    force=False,
+    silent=False,
+)
+
+if good:
+  ... load the new TF data ...
+```
+
+See `tf.convert.walker.CV.walk`.
+
+## Walking
+
+When you walk through the input data source, you'll encounter things
+that have to become slots, non-slot nodes, edges and features in the new data set.
+
+You issue these things by means of an *action method* from `cv`, such as
+`cv.slot()` or `cv.node(nodeType)`.
+
+When your action creates slots or non slot nodes,
+Text-Fabric will return you a reference to that node,
+that you can use later for more actions related to that node.
+
+```python
+curPara = cv.node('para')
+```
+
+To add features to nodes, use a `cv.feature()` action.
+It will apply to a node passed as argument.
+
+To add features to edges, issue a `cv.edge()` action.
+It will require two node arguments: the *from* node and the *to* node.
+
+There is always a set of current *embedder nodes*.
+When you create a slot node
+
+```python
+curWord = cv.slot()
+```
+
+then TF will link all current embedder nodes to the resulting slot.
+
+There are actions to add nodes to the set of embedder nodes,
+to remove them from it,
+and to add them again.
+
+## Dynamic Metadata
+
+When the director runs, you have already specified all your feature
+metadata, including the value types.
+
+But if some of that information is dependent on what you encounter in the
+data, you can do two things:
+
+(A) Run a preliminary pass over the data and gather the required information,
+before running the director.
+
+(B) Update the metadata later on
+by issuing `cv.meta()` actions from within your director, see below.
+
+## Action methods
+
+The `cv` class contains methods that are responsible for particular *actions*
+that steer the graph building:
+
+*   `tf.convert.walker.CV.slot`
+*   `tf.convert.walker.CV.node`
+*   `tf.convert.walker.CV.terminate`
+*   `tf.convert.walker.CV.resume`
+*   `tf.convert.walker.CV.feature`
+*   `tf.convert.walker.CV.edge`
+*   `tf.convert.walker.CV.meta`
+*   `tf.convert.walker.CV.occurs`
+*   `tf.convert.walker.CV.linked`
+*   `tf.convert.walker.CV.active`
+*   `tf.convert.walker.CV.activeTypes`
+*   `tf.convert.walker.CV.get` and `cv.get(feature, nf, nt)`
+*   `tf.convert.walker.CV.stop`
+
+!!! hint "Example"
+    Follow the
+    [conversion tutorial](https://nbviewer.jupyter.org/github/annotation/banks/blob/master/programs/convert.ipynb)
+
+    Or study a more involved example:
+    [Old Babylonian](https://github.com/Nino-cunei/oldbabylonian/blob/master/programs/tfFromATF.py)
 """
 
 import collections
@@ -124,10 +257,13 @@ class CV(object):
             The node type that acts as the type of the slots in the data set.
 
         oText: dict
-            The configuration information to be stored in the `otext` feature:
+            The configuration information to be stored in the `otext` feature
+            (see `tf.core.text`):
 
             * section types
             * section features
+            * structure types
+            * structure features
             * text formats
 
         generic: dict
@@ -446,6 +582,15 @@ class CV(object):
         If you have called `walk()` with `force=True`, indicating that the
         director must proceed after errors, then this stop command will cause termination
         nevertheless.
+
+        Parameters
+        ----------
+        msg: string
+            A message to display upon stopping.
+
+        Returns
+        -------
+        None
         """
 
         tmObj = self.TF.tmObj
@@ -468,6 +613,17 @@ class CV(object):
             cv.feature(n, key=value, ...)
 
         calls.
+
+        Parameters
+        ----------
+        None
+
+        Returns
+        -------
+        node reference: tuple
+            The node reference consists of a node type and a sequence number,
+            but normally you do not have to dig these out.
+            Just pass the tuple as a whole to actions that require a node argument.
         """
 
         curSeq = self.curSeq
@@ -493,20 +649,48 @@ class CV(object):
 
         return (nType, seq)
 
-    def node(self, nType):
+    def node(self, nType, slots=None):
         """Make a non-slot node and return the handle to it in `n`.
 
             n = cv.node(nodeType)
 
         You have to pass its *node type*, i.e. a string.
         Think of `sentence`, `paragraph`, `phrase`, `word`, `sign`, whatever.
-        Non slot nodes will be automatically added to the set of embedders.
+
+        There are two modes for this function:
+
+        * Auto: (`slots=None`):
+          Non slot nodes will be automatically added to the set of embedders.
+        * Explicit: (`slots=iterable`):
+          The slots in iterable will be assigned to this node and nothing else.
+          The node will not be added to the set of embedders.
+          Put otherwise: the node will be terminated after construction.
+          However: you could resume it later to add other slots.
 
         Remember that you can add features to the node by later
 
             cv.feature(n, key=value, ...)
 
         calls.
+
+        Parameters
+        ----------
+        nType: string
+            A node type, not the slot type
+        slots: iterable of int, optional `None`
+            The slots to assign to this node.
+            If left out, the node is left as an embedding node and
+            subsequent slots will be added to it automatically.
+            All slots in the iterable must have been generated before
+            by means of the `cv.slot()` action.
+
+        Returns
+        -------
+        node reference or None
+            If an error occurred, `None` is returned.
+            The node reference consists of a node type and a sequence number,
+            but normally you do not have to dig these out.
+            Just pass the tuple as a whole to actions that require a node argument.
         """
 
         slotType = self.slotType
@@ -526,7 +710,20 @@ class CV(object):
         node = (nType, seq)
 
         self._checkSecLevel(node, before=True)
-        curEmbedders.add(node)
+
+        if slots:
+            maxSlot = curSeq[slotType]
+
+            for s in slots:
+                if not 1 <= s <= maxSlot:
+                    errors[f'slot out of range in `cv.node(({nType}, {seq}))`'].append(f"{s}")
+                else:
+                    oslots = self.oslots
+                    oslots[node].add(s)
+
+            self.stats[self.T] += 1
+        else:
+            curEmbedders.add(node)
 
         return node
 
@@ -537,6 +734,15 @@ class CV(object):
 
         The node `n` will be removed from the set of current embedders.
         This `n` must be the result of a previous `cv.slot()` or `cv.node()` action.
+
+        Parameters
+        ----------
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+
+        Returns
+        -------
+        None
         """
 
         self.stats[self.T] += 1
@@ -549,12 +755,20 @@ class CV(object):
 
             cv.resume(n)
 
-
         If you resume a non-slot node, you add it again to the set of embedders.
         No new node will be created.
 
         If you resume a slot node, it will be added to the set of current embedders.
         No new slot will be created.
+
+        Parameters
+        ----------
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+
+        Returns
+        -------
+        None
         """
 
         curEmbedders = self.curEmbedders
@@ -575,15 +789,19 @@ class CV(object):
 
             cv.feature(n, name=value, ... , name=value)
 
+        Parameters
+        ----------
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+        **features: keyword arguments
+            The names and values of features to assign to this node.
 
-        The features are passed as a list of keyword arguments.
+        Returns
+        -------
+        None
 
-        These features are added to node `n`.
-        This `n` must be the result of a previous `cv.slot()` or `cv.node()` action.
-
-        **If a feature value is `None` it will not be added!**
-
-        The features are passed as a list of keyword arguments.
+        !!! caution "None values"
+            If a feature value is `None` it will not be added!
         """
 
         nodeFeatures = self.nodeFeatures
@@ -601,17 +819,25 @@ class CV(object):
 
             cv.edge(nf, nt, name=value, ... , name=value)
 
+        Parameters
+        ----------
+        nodeFrom, nodeTo: tuple
+            Two node references, obtained by one of the actions `slot` or `node`.
+        **features: keyword arguments
+            The names and values of features to assign to this edge
+            (i.e. pair of nodes).
 
-        You need to pass two nodes, `nf` (*from*) and `nt` (*to*).
-        Together these specify an edge, and the features will be applied
-        to this edge.
+        Returns
+        -------
+        None
 
-        You may pass values that are `None`, and a corresponding edge will be created.
-        If for all pairs `nf`, `nt` the value is `None`,
-        an edge without values will be created. For every `nf`, such a feature
-        essentially specifies a set of nodes `{ nt }`.
-
-        The features are passed as a list of keyword arguments.
+        !!! note "None values"
+            You may pass values that are `None`,
+            and a corresponding edge will be created.
+            If for all edges the value is `None`,
+            an edge without values will be created.
+            For every `nodeFrom`, such a feature
+            essentially specifies a set of nodes `{ nodeTo }`.
         """
 
         edgeFeatures = self.edgeFeatures
@@ -623,11 +849,10 @@ class CV(object):
             edgeFeatures[k][nodeFrom][nodeTo] = v
 
     def occurs(self, feat):
-        """Returns True if the feature with name `featureName` occurs in the
+        """Whether the feature `featureName` occurs in the resulting data so far.
 
             occurs = cv.occurs(featureName)
 
-        resulting data, else False.
         If you have assigned None values to a feature, that will count, i.e.
         that feature occurs in the data.
 
@@ -641,6 +866,15 @@ class CV(object):
         such is feature is declared but not used.
         At the end of your director you can remove unused features
         conditionally, using this function.
+
+        Parameters
+        ----------
+        feat: string
+            The name of a feature
+
+        Returns
+        -------
+        boolean
         """
 
         nodeFeatures = self.nodeFeatures
@@ -654,16 +888,22 @@ class CV(object):
 
             cv.meta(feature, name=value, ... , name=value)
 
+        Parameters
+        ----------
+        feat: string
+            The name of a feature
+        **metaData: pairs of name and value
+            If a `value` is `None`, that `name` will be deleted from the
+            metadata fields of the feature.
+            A bare `cv.meta(feature)` will remove the all metadata from the feature.
+            If you modify the field `valueType` of a feature, that feature will be
+            added or removed from the set of `intFeatures`.
+            It will be checked whether you specify either `int` or `str`.
 
-        `feature` is the feature name.
 
-        If a `value` is `None`, that `name` will be deleted from the metadata fields.
-
-        A bare `cv.meta(feature)` will remove the feature from the metadata.
-
-        If you modify the field `valueType` of a feature, that feature will be added
-        or removed from the set of `intFeatures`.
-        It will be checked whether you specify either `int` or `str`.
+        Returns
+        -------
+        None
         """
 
         errors = self.errors
@@ -710,31 +950,58 @@ class CV(object):
         to do so, you have to detect that a node is still unlinked.
 
         This action is precisely meant for that.
+
+        Parameters
+        ----------
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+
+        Returns
+        -------
+        boolean
         """
 
         oslots = self.oslots
         return tuple(oslots.get(node, []))
 
     def active(self, node):
-        """Returns whether the node `n` is currently active, i.e. in the set
+        """Returns whether a node is currently active.
+
+        Active nodes are the nodes in the set of current embedders.
 
             isActive = cv.active(n)
 
-        of embedders.
-
         If you construct your nodes in a very dynamic way, it might be
         hard to keep track for each node whether it has been created, terminated,
         or resumed, in other words, whether it is active or not.
 
-        This action is provides a direct and precise way to know whether a node is active.
+        This action is provides a direct and precise way to know
+        whether a node is active.
+
+        Parameters
+        ----------
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+
+        Returns
+        -------
+        boolean
         """
 
         return node in self.curEmbedders
 
     def activeTypes(self):
-        """Returns the node types of the currently active nodes, i.e. the embedders.
+        """The node types of the currently active nodes, i.e. the embedders.
 
             nTypes = cv.activeTypes()
+
+        Parameters
+        ----------
+        None
+
+        Returns
+        -------
+        set
         """
 
         return {node[0] for node in self.curEmbedders}
@@ -750,6 +1017,21 @@ class CV(object):
         For node features, `n` is the node which carries the value.
 
         For edge features, `nf, nt` is the pair of from-to nodes which carries the value.
+
+        Parameters
+        ----------
+        feature: string
+            The name of a feature
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+            The node in question when retrieving the value of a node feature.
+        nodeFrom, nodeTo: tuple
+            Two node references, obtained by one of the actions `slot` or `node`.
+            The nodes in question when retrieving the value of an edge feature.
+
+        Returns
+        -------
+        string or integer
         """
 
         errors = self.errors
