@@ -1,5 +1,22 @@
 """
-.. include:: ../docs/main/fabric.md
+# Fabric
+
+The main class that works the core API is `tf.fabric.Fabric`.
+
+It is responsible for feature data loading and saving.
+
+!!! note "Tutorial"
+    The tutorials for specific [annotated corpora](https://github.com/annotation)
+    put the Text-Fabric API on show for vastly different corpora.
+
+!!! note "Generic API versus apps"
+    This is the API of Text-Fabric in general.
+    Text-Fabric has no baked in knowledge of particular corpora.
+
+    However, Text-Fabric comes with several *apps* that make working
+    with specific `tf.about.corpora` easier.
+    The result of those apps (which may only exist of a (zero-content) *config.yaml* file,
+    is available through the advanced API: `A`, see `tf.app`.
 """
 
 import os
@@ -52,7 +69,16 @@ PRECOMPUTE = (
     (False, "__levDown__", levDown, (WARP[0], "__levUp__", "__rank__")),
     (False, "__boundary__", boundary, WARP[0:2] + ("__rank__",)),
     (True, "__sections__", sections, WARP + ("__levUp__", "__levels__")),
-    (True, "__structure__", structure, WARP + ("__rank__", "__levUp__",)),
+    (
+        True,
+        "__structure__",
+        structure,
+        WARP
+        + (
+            "__rank__",
+            "__levUp__",
+        ),
+    ),
 )
 """Precomputation steps.
 
@@ -201,7 +227,7 @@ Api reference : {APIREF}
             self.locations.append(expandDir(self, loc))
 
         self.locationRep = "\n\t".join(
-            "\n\t".join(f"{l}/{f}" for f in self.modules) for l in self.locations
+            "\n\t".join(f"{lc}/{f}" for f in self.modules) for lc in self.locations
         )
         self.featuresRequested = []
         self.features = {}
@@ -426,7 +452,10 @@ Api reference : {APIREF}
             dest.add(fName)
         info(
             "Feature overview: {} for nodes; {} for edges; {} configs; {} computed".format(
-                len(nodes), len(edges), len(configs), len(computeds),
+                len(nodes),
+                len(edges),
+                len(configs),
+                len(computeds),
             )
         )
         self.featureSets = dict(
@@ -666,7 +695,8 @@ Api reference : {APIREF}
                             otypeData.get(n, "_UNKNOWN_"), []
                         ).append(n)
                     for (nType, nodes) in sorted(
-                        unmappedByType.items(), key=lambda x: (-len(x[1]), x[0]),
+                        unmappedByType.items(),
+                        key=lambda x: (-len(x[1]), x[0]),
                     ):
                         error(f"--- unmapped {nType:<10} : {makeExamples(nodes)}")
                     good = False
@@ -842,7 +872,8 @@ Api reference : {APIREF}
         self._getWriteLoc()
         info(
             "{} features found and {} ignored".format(
-                len(tfFiles), sum(len(x) for x in self.featuresIgnored.values()),
+                len(tfFiles),
+                sum(len(x) for x in self.featuresIgnored.values()),
             ),
             tm=False,
         )
@@ -1017,21 +1048,13 @@ Api reference : {APIREF}
                         if fName in WARP:
                             continue
                         elif fObj.isEdge:
-                            if not hasattr(api.E, fName):
-                                setattr(
-                                    api.E,
-                                    fName,
-                                    EdgeFeature(
-                                        api, fObj.metaData, fObj.data, fObj.edgeValues
-                                    ),
-                                )
+                            apiFobj = EdgeFeature(
+                                api, fObj.metaData, fObj.data, fObj.edgeValues
+                            )
+                            setattr(api.E, fName, apiFobj)
                         else:
-                            if not hasattr(api.F, fName):
-                                setattr(
-                                    api.F,
-                                    fName,
-                                    NodeFeature(api, fObj.metaData, fObj.data),
-                                )
+                            apiFobj = NodeFeature(api, fObj.metaData, fObj.data)
+                            setattr(api.F, fName, apiFobj)
                     else:
                         if fName in WARP or fName in self.textFeatures:
                             continue
